@@ -6,6 +6,7 @@ const Gtk = imports.gi.Gtk;
 const Shell = imports.gi.Shell;
 const Lang = imports.lang;
 const Signals = imports.signals;
+const Meta = imports.gi.Meta;
 const St = imports.gi.St;
 const Mainloop = imports.mainloop;
 const Gettext = imports.gettext.domain('gnome-shell');
@@ -22,6 +23,7 @@ const Tweener = imports.ui.tweener;
 const Workspace = imports.ui.workspace;
 const Params = imports.misc.params;
 
+const MAX_APPLICATION_WORK_MILLIS = 75;
 const MENU_POPUP_TIMEOUT = 600;
 const SCROLL_TIME = 0.1;
 
@@ -34,6 +36,8 @@ AlphabeticalView.prototype = {
         this._grid = new IconGrid.IconGrid({ xAlign: St.Align.START });
         this._appSystem = Shell.AppSystem.get_default();
 
+        this._pendingAppLaterId = 0;
+        this._apps = [];
         this._filterApp = null;
 
         let box = new St.BoxLayout({ vertical: true });
@@ -86,10 +90,14 @@ AlphabeticalView.prototype = {
         if (vfade)
             offset = vfade.fade_offset;
 
-        if (icon.y < value + offset)
-            value = Math.max(0, icon.y - offset);
-        else if (icon.y + icon.height > value + pageSize - offset)
-            value = Math.min(upper, icon.y + icon.height + offset - pageSize);
+        // If this gets called as part of a right-click, the actor
+        // will be needs_allocation, and so "icon.y" would return 0
+        let box = icon.get_allocation_box();
+
+        if (box.y1 < value + offset)
+            value = Math.max(0, box.y1 - offset);
+        else if (box.y2 > value + pageSize - offset)
+            value = Math.min(upper, box.y2 + offset - pageSize);
         else
             return;
 
@@ -105,6 +113,30 @@ AlphabeticalView.prototype = {
             this._apps[i].actor.visible = filter(this._apps[i]._appInfo);
     },
 
+    // Create actors for the applications in an idle to avoid blocking
+    // for too long; see bug 647778
+    _addPendingApps: function() {
+        let i;
+        let startTimeMillis = new Date().getTime();
+        for (i = 0; i < this._pendingAppIds.length; i++) {
+            let id = this._pendingAppIds[i];
+            this._addApp(this._pendingApps[id]);
+
+            let currentTimeMillis = new Date().getTime();
+            if (currentTimeMillis - startTimeMillis > MAX_APPLICATION_WORK_MILLIS)
+                break;
+        }
+        this._pendingAppIds.splice(0, i);
+        if (this._pendingAppIds.length > 0) {
+            return true;
+        } else {
+            this._pendingAppLaterId = 0;
+            this._pendingAppIds = null;
+            this._pendingApps = null;
+            return false;
+        }
+    },
+
     refresh: function(apps) {
         let ids = [];
         for (let i in apps)
@@ -115,9 +147,12 @@ AlphabeticalView.prototype = {
 
         this._removeAll();
 
-        for (let i = 0; i < ids.length; i++) {
-            this._addApp(apps[ids[i]]);
-        }
+        this._pendingAppIds = ids;
+        this._pendingApps = apps;
+        if (this._pendingAppLaterId)
+            Meta.later_remove(this._pendingAppLaterId);
+        this._pendingAppLaterId = Meta.later_add(Meta.LaterType.BEFORE_REDRAW,
+                                                 Lang.bind(this, this._addPendingApps));
     }
 };
 
@@ -220,7 +255,6 @@ ViewByCategories.prototype = {
 
         let sections = this._appSystem.get_sections();
         this._apps = apps;
-        this._view.refresh(apps);
 
         /* Translators: Filter to display all applications */
         this._addFilter(_("All"), -1);
@@ -232,6 +266,7 @@ ViewByCategories.prototype = {
             this._addFilter(sections[i], i);
 
         this._selectCategory(-1);
+        this._view.refresh(apps);
 
         if (this._focusDummy) {
             let focused = this._focusDummy.has_key_focus();
@@ -309,7 +344,7 @@ BaseAppSearchProvider.prototype = {
                                         timestamp: null });
 
         let app = this._appSys.get_app(id);
-        app.open_new_window(params.workspace ? params.workspace.get_index() : -1);
+        app.open_new_window(params.workspace ? params.workspace.index() : -1);
     }
 };
 
@@ -412,6 +447,10 @@ AppWellIcon.prototype = {
                 this._removeMenuTimeout();
                 Main.overview.beginItemDrag(this);
             }));
+        this._draggable.connect('drag-cancelled', Lang.bind(this,
+            function () {
+                Main.overview.cancelledItemDrag(this);
+            }));
         this._draggable.connect('drag-end', Lang.bind(this,
             function () {
                Main.overview.endItemDrag(this);
@@ -505,6 +544,8 @@ AppWellIcon.prototype = {
             this._menuManager.addMenu(this._menu);
         }
 
+        this.actor.set_hover(true);
+        this.actor.show_tooltip();
         this._menu.popup();
 
         return false;
@@ -568,6 +609,9 @@ AppIconMenu.prototype = {
 
         PopupMenu.PopupMenu.prototype._init.call(this, source.actor, 0.5, side, 0);
 
+        // We want to keep the item hovered while the menu is up
+        this.blockSourceEvents = true;
+
         this._source = source;
 
         this.connect('activate', Lang.bind(this, this._onActivate));
