@@ -13,12 +13,6 @@ const Params = imports.misc.params;
 // normal mode (ie, outside the Overview), that surrounds the main
 // workspace content.
 
-const Visibility = {
-    FULL:       1,
-    FULLSCREEN: 2,
-    OVERVIEW:   3
-};
-
 const defaultParams = {
     visibleInOverview: false,
     visibleInFullscreen: false,
@@ -37,12 +31,13 @@ Chrome.prototype = {
         Main.uiGroup.add_actor(this.actor);
         this.actor.connect('allocate', Lang.bind(this, this._allocated));
 
-        this._inFullscreen = false;
+        this._monitors = [];
         this._inOverview = false;
-        this.visibility = Visibility.FULL;
 
         this._trackedActors = [];
 
+        global.screen.connect('monitors-changed',
+                              Lang.bind(this, this._monitorsChanged));
         global.screen.connect('restacked',
                               Lang.bind(this, this._windowsRestacked));
 
@@ -55,6 +50,8 @@ Chrome.prototype = {
         Main.overview.connect('hidden',
                              Lang.bind(this, this._overviewHidden));
 
+        this._updateMonitors();
+        this._updateFullscreen();
         this._queueUpdateRegions();
     },
 
@@ -194,24 +191,12 @@ Chrome.prototype = {
             let actorData = this._trackedActors[i];
             if (this._inOverview && !actorData.visibleInOverview)
                 this.actor.set_skip_paint(actorData.actor, true);
-            else if (!this._inOverview && this._inFullscreen && !actorData.visibleInFullscreen)
+            else if (!this._inOverview && !actorData.visibleInFullscreen &&
+                     this._findMonitorForActor(actorData.actor).inFullscreen)
                 this.actor.set_skip_paint(actorData.actor, true);
             else
                 this.actor.set_skip_paint(actorData.actor, false);
         }
-
-        let newVisibility;
-        if (this._inOverview)
-            newVisibility = Visibility.OVERVIEW;
-        else if (this._inFullscreen)
-            newVisibility = Visibility.FULLSCREEN;
-        else
-            newVisibility = Visibility.FULL;
-
-        if (newVisibility != this.visibility) {
-            this.visibility = newVisibility;
-            this.emit('visibility-changed', this.visibility);
-        }
     },
 
     _overviewShowing: function() {
@@ -226,15 +211,77 @@ Chrome.prototype = {
         this._queueUpdateRegions();
     },
 
+    _updateMonitors: function() {
+        let monitors = global.get_monitors();
+        let primary = global.get_primary_monitor();
+        this._monitors = monitors;
+        for (let i = 0; i < monitors.length; i++) {
+            let monitor = monitors[i];
+            if (monitor.x == primary.x &&
+                monitor.y == primary.y &&
+                monitor.width == primary.width &&
+                monitor.height == primary.height)
+                this._primaryMonitor = monitor;
+        }
+    },
+
+    _findMonitorForRect: function(x, y, w, h) {
+        // First look at what monitor the center of the rectangle is at
+        let cx = x + w/2;
+        let cy = y + h/2;
+        for (let i = 0; i < this._monitors.length; i++) {
+            let monitor = this._monitors[i];
+            if (cx >= monitor.x && cx < monitor.x + monitor.width &&
+                cy >= monitor.y && cy < monitor.y + monitor.height)
+                return monitor;
+        }
+        // If the center is not on a monitor, return the first overlapping monitor
+        for (let i = 0; i < this._monitors.length; i++) {
+            let monitor = this._monitors[i];
+            if (x + w > monitor.x && x < monitor.x + monitor.width &&
+                y + h > monitor.y && y < monitor.y + monitor.height)
+                return monitor;
+        }
+        // otherwise on no monitor
+        return null;
+    },
+
+    _findMonitorForWindow: function(window) {
+        return this._findMonitorForRect(window.x, window.y, window.width, window.height);
+    },
+
+    // This call guarantees that we return some monitor to simplify usage of it
+    // In practice all tracked actors should be visible on some monitor anyway
+    _findMonitorForActor: function(actor) {
+        let [x, y] = actor.get_transformed_position();
+        let [w, h] = actor.get_transformed_size();
+        let monitor = this._findMonitorForRect(x, y, w, h);
+        if (monitor)
+            return monitor;
+        return this._primaryMonitor; // Not on any monitor, pretend its on the primary
+    },
+
+    _monitorsChanged: function() {
+        this._updateMonitors();
+
+        // Update everything that depends on monitor positions
+        this._updateFullscreen();
+        this._updateVisibility();
+        this._queueUpdateRegions();
+    },
+
     _queueUpdateRegions: function() {
         if (!this._updateRegionIdle)
             this._updateRegionIdle = Mainloop.idle_add(Lang.bind(this, this._updateRegions),
                                                        Meta.PRIORITY_BEFORE_REDRAW);
     },
 
-    _windowsRestacked: function() {
+    _updateFullscreen: function() {
         let windows = Main.getWindowActorsForWorkspace(global.screen.get_active_workspace_index());
-        let primary = global.get_primary_monitor();
+
+        // Reset all monitors to not fullscreen
+        for (let i = 0; i < this._monitors.length; i++)
+            this._monitors[i].inFullscreen = false;
 
         // The chrome layer should be visible unless there is a window
         // with layer FULLSCREEN, or a window with layer
@@ -248,42 +295,51 @@ Chrome.prototype = {
 
         // @windows is sorted bottom to top.
 
-        let wasInFullscreen = this._inFullscreen;
-        this._inFullscreen = false;
         for (let i = windows.length - 1; i > -1; i--) {
-            let layer = windows[i].get_meta_window().get_layer();
-
-            // There are 3 cases we check here for:
-            // 1.) Monitor sized window
-            // 2.) Window with a position somewhere on the primary screen having the _NET_WM_FULLSCREEN flag set
-            // 3.) Window that is partly off screen (tries to hide its decorations) which might have negative coords
-            // We check for 1.) and 2.) by checking if the upper right corner is on the primary monitor, but avoid the case
-            // where it overlaps with the secondary screen (like window.x + window.width == primary.x + primary.width)
-            // For 3.) we just ignore negative values as they don't really make sense
+            let window = windows[i];
+            let layer = window.get_meta_window().get_layer();
 
             if (layer == Meta.StackLayer.FULLSCREEN) {
-                if (Math.max(windows[i].x, 0) >= primary.x && Math.max(windows[i].x, 0) < primary.x + primary.width &&
-                    Math.max(windows[i].y, 0) >= primary.y && Math.max(windows[i].y, 0) < primary.y + primary.height) {
-                        this._inFullscreen = true;
-                        break;
-                }
+                let monitor = this._findMonitorForWindow(window);
+                if (monitor)
+                    monitor.inFullscreen = true;
             }
             if (layer == Meta.StackLayer.OVERRIDE_REDIRECT) {
-                if (windows[i].x <= primary.x &&
-                    windows[i].x + windows[i].width >= primary.x + primary.width &&
-                    windows[i].y <= primary.y &&
-                    windows[i].y + windows[i].height >= primary.y + primary.height) {
-                    this._inFullscreen = true;
-                    break;
-                }
+                let monitor = this._findMonitorForWindow(window);
+                if (monitor &&
+                    window.x <= monitor.x &&
+                    window.x + window.width >= monitor.x + monitor.width &&
+                    window.y <= monitor.y &&
+                    window.y + window.height >= monitor.y + monitor.height)
+                    monitor.inFullscreen = true;
             } else
                 break;
         }
+    },
 
-        if (this._inFullscreen != wasInFullscreen) {
+    _windowsRestacked: function() {
+        let wasInFullscreen = [];
+        for (let i = 0; i < this._monitors.length; i++)
+            wasInFullscreen[i] = this._monitors[i].inFullscreen;
+
+        this._updateFullscreen();
+
+        let changed = false;
+        for (let i = 0; i < wasInFullscreen.length; i++) {
+            if (wasInFullscreen[i] != this._monitors[i].inFullscreen) {
+                changed = true;
+                break;
+            }
+        }
+        if (changed) {
             this._updateVisibility();
             this._queueUpdateRegions();
         }
+
+        // Figure out where the pointer is in case we lost track of
+        // it during a grab. (In particular, if a trayicon popup menu
+        // is dismissed, see if we need to close the message tray.)
+        global.sync_pointer();
     },
 
     _updateRegions: function() {
@@ -312,41 +368,75 @@ Chrome.prototype = {
             if (!actorData.affectsStruts)
                 continue;
 
+            // Limit struts to the size of the screen
+            let x1 = Math.max(x, 0);
+            let x2 = Math.min(x + w, global.screen_width);
+            let y1 = Math.max(y, 0);
+            let y2 = Math.min(y + h, global.screen_height);
+
+            // NetWM struts are not really powerful enought to handle
+            // a multi-monitor scenario, they only describe what happens
+            // around the outer sides of the full display region. However
+            // it can describe a partial region along each side, so
+            // we can support having the struts only affect the
+            // primary monitor. This should be enough as we only have
+            // chrome affecting the struts on the primary monitor so
+            // far.
+            //
             // Metacity wants to know what side of the screen the
             // strut is considered to be attached to. If the actor is
             // only touching one edge, or is touching the entire
-            // width/height of one edge, then it's obvious which side
-            // to call it. If it's in a corner, we pick a side
+            // border of the primary monitor, then it's obvious which
+            // side to call it. If it's in a corner, we pick a side
             // arbitrarily. If it doesn't touch any edges, or it spans
             // the width/height across the middle of the screen, then
             // we don't create a strut for it at all.
             let side;
-            if (w >= global.screen_width) {
-                if (y <= 0)
+            let primary = this._primaryMonitor;
+            if (x1 <= primary.x && x2 >= primary.x + primary.width) {
+                if (y1 <= primary.y)
                     side = Meta.Side.TOP;
-                else if (y + h >= global.screen_height)
+                else if (y2 >= primary.y + primary.height)
                     side = Meta.Side.BOTTOM;
                 else
                     continue;
-            } else if (h >= global.screen_height) {
-                if (x <= 0)
+            } else if (y1 <= primary.y && y2 >= primary.y + primary.height) {
+                if (x1 <= 0)
                     side = Meta.Side.LEFT;
-                else if (x + w >= global.screen_width)
+                else if (x2 >= global.screen_width)
                     side = Meta.Side.RIGHT;
                 else
                     continue;
-            } else if (x <= 0)
+            } else if (x1 <= 0)
                 side = Meta.Side.LEFT;
-            else if (y <= 0)
+            else if (y1 <= 0)
                 side = Meta.Side.TOP;
-            else if (x + w >= global.screen_width)
+            else if (x2 >= global.screen_width)
                 side = Meta.Side.RIGHT;
-            else if (y + h >= global.screen_height)
+            else if (y2 >= global.screen_height)
                 side = Meta.Side.BOTTOM;
             else
                 continue;
 
-            let strut = new Meta.Strut({ rect: rect, side: side });
+            // Ensure that the strut rects goes all the way to the screen edge,
+            // as this really what mutter expects.
+            switch (side) {
+            case Meta.Side.TOP:
+                y1 = 0;
+                break;
+            case Meta.Side.BOTTOM:
+                y2 = global.screen_height;
+                break;
+            case Meta.Side.LEFT:
+                x1 = 0;
+                break;
+            case Meta.Side.RIGHT:
+                x2 = global.screen_width;
+                break;
+            }
+
+            let strutRect = new Meta.Rectangle({ x: x1, y: y1, width: x2 - x1, height: y2 - y1});
+            let strut = new Meta.Strut({ rect: strutRect, side: side });
             struts.push(strut);
         }
 
