@@ -19,7 +19,8 @@ const STANDARD_TRAY_ICON_IMPLEMENTATIONS = {
     'a11y-keyboard': 'a11y',
     'kbd-scrolllock': 'keyboard',
     'kbd-numlock': 'keyboard',
-    'kbd-capslock': 'keyboard'
+    'kbd-capslock': 'keyboard',
+    'ibus-ui-gtk': 'input-method'
 };
 
 function StatusIconDispatcher() {
