@@ -4,8 +4,7 @@
 
 #include <clutter/clutter.h>
 #include <gio/gio.h>
-
-#include "window.h"
+#include <meta/window.h>
 
 G_BEGIN_DECLS
 
@@ -44,9 +43,11 @@ gboolean shell_app_is_transient (ShellApp *app);
 
 void shell_app_activate_window (ShellApp *app, MetaWindow *window, guint32 timestamp);
 
-void shell_app_activate (ShellApp *app);
+void shell_app_activate (ShellApp      *app,
+                         int            workspace);
 
-void shell_app_open_new_window (ShellApp *app);
+void shell_app_open_new_window (ShellApp *app,
+                                int       workspace);
 
 ShellAppState shell_app_get_state (ShellApp *app);
 
@@ -56,6 +57,8 @@ guint shell_app_get_n_windows (ShellApp *app);
 
 GSList *shell_app_get_windows (ShellApp *app);
 
+GSList *shell_app_get_pids (ShellApp *app);
+
 gboolean shell_app_is_on_workspace (ShellApp *app, MetaWorkspace *workspace);
 
 int shell_app_compare (ShellApp *app, ShellApp *other);
