@@ -54,6 +54,23 @@ shell_stage_input_mode_get_type(void) {
     }
   return enum_type_id;
 }
+/* enumerations from "shell-mobile-providers.h" */
+#include "shell-mobile-providers.h"
+GType
+shell_mobile_access_method_type_get_type(void) {
+  static GType enum_type_id = 0;
+  if (G_UNLIKELY (!enum_type_id))
+    {
+      static const GEnumValue values[] = {
+        { SHELL_MOBILE_ACCESS_METHOD_TYPE_UNKNOWN, "SHELL_MOBILE_ACCESS_METHOD_TYPE_UNKNOWN", "unknown" },
+        { SHELL_MOBILE_ACCESS_METHOD_TYPE_GSM, "SHELL_MOBILE_ACCESS_METHOD_TYPE_GSM", "gsm" },
+        { SHELL_MOBILE_ACCESS_METHOD_TYPE_CDMA, "SHELL_MOBILE_ACCESS_METHOD_TYPE_CDMA", "cdma" },
+        { 0, NULL, NULL }
+      };
+      enum_type_id = g_enum_register_static("ShellMobileAccessMethodType", values);
+    }
+  return enum_type_id;
+}
 
 /* Generated data ends here */
 
