@@ -225,23 +225,6 @@ get_appid_from_window (MetaWindow  *window)
   return appid_guess;
 }
 
-/**
- * window_is_tracked:
- *
- * We don't attempt to associate override-redirect windows with applications
- * at all, since there's no reason to do so yet.
- *
- * Returns: %TRUE iff we want to scan this window for application association
- */
-static gboolean
-window_is_tracked (MetaWindow *window)
-{
-  if (meta_window_is_override_redirect (window))
-    return FALSE;
-
-  return TRUE;
-}
-
 /**
  * shell_window_tracker_is_window_interesting:
  *
@@ -446,12 +429,14 @@ get_app_for_window (ShellWindowTracker    *monitor,
         }
     }
 
-  result = get_app_from_window_pid (monitor, window);
+  /* Check if the app's WM_CLASS specifies an app; this is
+   * canonical if it does.
+   */
+  result = get_app_from_window_wmclass (window);
   if (result != NULL)
     return result;
 
-  /* Check if the app's WM_CLASS specifies an app */
-  result = get_app_from_window_wmclass (window);
+  result = get_app_from_window_pid (monitor, window);
   if (result != NULL)
     return result;
 
@@ -529,7 +514,7 @@ track_window (ShellWindowTracker *self,
 {
   ShellApp *app;
 
-  if (!window_is_tracked (window))
+  if (!shell_window_tracker_is_window_interesting (window))
     return;
 
   app = get_app_for_window (self, window);
@@ -539,10 +524,6 @@ track_window (ShellWindowTracker *self,
   /* At this point we've stored the association from window -> application */
   g_hash_table_insert (self->window_to_app, window, app);
 
-  /* However, only put interesting windows in the window list for an app. */
-  if (!shell_window_tracker_is_window_interesting (window))
-    return;
-
   if (shell_app_is_transient (app))
     {
       /* For a transient application, it's possible one of our title regexps
@@ -907,7 +888,7 @@ on_focus_window_changed (MetaDisplay        *display,
   ShellApp *new_focus_app;
 
   new_focus_win = meta_display_get_focus_window (display);
-  new_focus_app = new_focus_win ? g_hash_table_lookup (tracker->window_to_app, new_focus_win) : NULL;
+  new_focus_app = new_focus_win ? shell_window_tracker_get_window_app (tracker, new_focus_win) : NULL;
 
   set_focus_app (tracker, new_focus_app);
 }
